@@ -17,18 +17,19 @@
 import collections
 import dataclasses
 import functools
-from typing import (Any, Callable, Generic, Iterable, List, Mapping, Optional,
-                    Sequence, Tuple, TypeVar, Union)
+from typing import (Any, Callable, Dict, Generic, Iterable, List, Mapping,
+                    Optional, Sequence, Tuple, TypeVar, Union)
 import warnings
 
 from . import axes_scan
+from . import meta
 from flax import traceback_util
 from .frozen_dict import freeze
 from .frozen_dict import unfreeze
 import jax
 from jax import random
 from .scope import (CollectionFilter, DenyList, PRNGSequenceFilter,  # pylint: disable=g-multiple-import
-                    Scope, group_collections, in_filter,
+                    Filter, Scope, group_collections, in_filter,
                     intersect_filters, is_filter_empty, subtract_filters,
                     union_filters)
 
@@ -148,7 +149,7 @@ def pack(fn: Callable[..., Any],
       for inner_scope in inner_scopes:
         inner_scope.invalidate()
       inner_scopes = []
-      mutable = False
+      mutable: Filter = False
       for out_filter in out_variable_filters:
         mutable = union_filters(mutable, out_filter)
       # could be () in the edge case where no rngs or variable_groups are lifted
@@ -523,7 +524,8 @@ def vmap(fn: Callable[..., Any],
          out_axes=0,
          axis_size: Optional[int] = None,
          axis_name: Optional[str] = None,
-         spmd_axis_name: Optional[str] = None) -> Callable[..., Any]:
+         spmd_axis_name: Optional[str] = None,
+         metadata_params: Dict[Any, Any] = {}) -> Callable[..., Any]:
   """A lifted version of ``jax.vmap``.
 
   See ``jax.vmap`` for the unlifted batch transform in Jax.
@@ -574,6 +576,8 @@ def vmap(fn: Callable[..., Any],
     spmd_axis_name: Axis name added to any pjit sharding constraints appearing
       in `fn`. See also
       https://github.com/google/flax/blob/main/flax/linen/partitioning.py.
+    metadata_params: arguments dict passed to AxisMetadata instances in the
+      variable tree.
 
   Returns:
     A vectorized version of the input scope function.
@@ -611,6 +615,15 @@ def vmap(fn: Callable[..., Any],
         tree_map_rngs(split_fn, rng_group) if split else rng_group
         for rng_group, split in zip(rng_groups, rng_splits))
 
+    new_variable_groups = []
+    for var_group, axis in zip(variable_groups, variable_in_axes):
+      if axis is not None:
+        new_variable_groups.append(meta.remove_axis(
+            var_group, axis, metadata_params))
+      else:
+        new_variable_groups.append(var_group)
+    variable_groups = tuple(new_variable_groups)
+
     @functools.partial(
         jax.vmap,
         in_axes=(variable_in_axes, rng_axes, in_axes),
@@ -624,7 +637,15 @@ def vmap(fn: Callable[..., Any],
       y = fn(scope, *args)
       return y, repack_fn(scope)
 
-    return mapped(variable_groups, rng_groups, args)
+    y, vars_out = mapped(variable_groups, rng_groups, args)
+    new_vars_out = []
+    for var_group, axis in zip(vars_out, variable_out_axes):
+      if axis is not None:
+        new_vars_out.append(meta.add_axis(var_group, axis, metadata_params))
+      else:
+        new_vars_out.append(var_group)
+    vars_out = tuple(new_vars_out)
+    return y, vars_out
 
   return pack(
       inner, variable_in_groups, variable_out_groups, rng_groups,
@@ -634,7 +655,6 @@ def vmap(fn: Callable[..., Any],
 ScanAxis = int
 InOutScanAxis = Union[ScanAxis, In[ScanAxis], Out[ScanAxis]]
 
-
 def scan(fn: Callable[..., Any],
          variable_axes: Mapping[CollectionFilter, InOutScanAxis] = {},
          variable_broadcast: CollectionFilter = False,
@@ -645,6 +665,7 @@ def scan(fn: Callable[..., Any],
          reverse: bool = False,
          unroll: int = 1,
          data_transform: Optional[Callable[..., Any]] = None,
+         metadata_params: Dict[Any, Any] = {},
          ) -> Callable[..., Any]:
   """A lifted version of ``jax.lax.scan``.
 
@@ -708,6 +729,8 @@ def scan(fn: Callable[..., Any],
       iteration of a loop (default: 1).
     data_transform: optional function to transform raw variable and rng groups,
       intended for inline SPMD annotations.
+    metadata_params: arguments dict passed to AxisMetadata instances in the
+      variable tree.
 
   Returns:
     The scan function with the signature
@@ -777,8 +800,16 @@ def scan(fn: Callable[..., Any],
     broadcast_vars = variable_groups[0]
     carry_vars = variable_groups[1]
     scan_vars = variable_groups[2:]
+    new_scan_vars = []
+    for scan_group, axis in zip(scan_vars, variable_in_axes):
+      new_scan_vars.append(meta.remove_axis(scan_group, axis, metadata_params))
     broadcast_vars, (carry_vars, c), (ys, scan_vars) = scanned(
-        broadcast_vars, (carry_vars, init), scan_vars, rng_groups, args)
+        broadcast_vars, (carry_vars, init), tuple(new_scan_vars),
+        rng_groups, args)
+    new_scan_vars = []
+    for scan_group, axis in zip(scan_vars, variable_out_axes):
+      new_scan_vars.append(meta.add_axis(scan_group, axis, metadata_params))
+    scan_vars = tuple(new_scan_vars)
     out_vars = (broadcast_vars, carry_vars,) + scan_vars
     return (c, ys), out_vars
 
@@ -1264,8 +1295,8 @@ def jit(fn: Callable[..., Any],
   # Close over scope_fn & repack_fn to avoid recompilation
   # this is impure but we use the fingerprint arg to differentiate between cases
   # where scope_fn or repack_fn actually produce non-identical results.
-  scope_fn = None  # type: Callable
-  repack_fn = None  # type: Callable
+  scope_fn = None  # type: Optional[Callable]
+  repack_fn = None  # type: Optional[Callable]
   @functools.partial(jax.jit,
                      static_argnums=static_argnums,
                      donate_argnums=donate_argnums,
