@@ -23,7 +23,7 @@ from typing import (Any, Callable, Dict, Generic, Iterable, Mapping, Optional,
                     Sequence, Set, Tuple, TypeVar, Union)
 
 from flax.ids import uuid
-from flax import config
+from flax import configurations as config
 from flax import errors
 from flax import struct
 from flax import traceback_util
@@ -31,10 +31,12 @@ from .frozen_dict import freeze
 from .frozen_dict import FrozenDict
 from .frozen_dict import unfreeze
 from . import tracers
+from . import meta
 import jax
 from jax import config as jax_config
 from jax import numpy as jnp
 from jax import random
+from jax import tree_util
 import numpy as np
 
 traceback_util.register_exclusion(__file__)
@@ -112,7 +114,7 @@ def _legacy_rng_fold_in(rng: PRNGKey, data: Iterable[PRNGFoldable]) -> PRNGKey:
       m.update(x.encode('utf-8'))
       d = m.digest()
       hash_int = int.from_bytes(d[:4], byteorder='big')
-      rng = random.fold_in(rng, jnp.uint32(hash_int))
+      rng = random.fold_in(rng, jnp.uint32(hash_int))  # type: ignore
     elif isinstance(x, int):
       rng = random.fold_in(rng, x)
     else:
@@ -146,7 +148,7 @@ def _fold_in_static(rng: PRNGKey,
       raise ValueError(f'Expected int or string, got: {x}')
   d = m.digest()
   hash_int = int.from_bytes(d[:4], byteorder='big')
-  return random.fold_in(rng, jnp.uint32(hash_int))
+  return random.fold_in(rng, jnp.uint32(hash_int)) # type: ignore
 
 
 def is_filter_empty(filter_like: Filter) -> bool:
@@ -335,27 +337,42 @@ class Variable(Generic[T]):
   content and can be assigned to for mutation.
   """
 
-  def __init__(self, scope: 'Scope', collection: str, name: str):
+  def __init__(
+      self, scope: 'Scope', collection: str, name: str, unbox: bool):
     """Initializes a variable.
 
     Args:
       scope: The scope in which the variable is stored.
       collection: The collection of the variable (e.g., "params").
       name: The name of the variable (e.g., "dense").
+      unbox: Whether to unbox boxed values with metadata.
     """
     self._id = uuid()
     self.scope = scope
     self.collection = collection
     self.name = name
+    self.unbox = unbox
 
   @property
   def value(self) -> T:
     """Returns the value of this Variable."""
-    return self.scope.get_variable(self.collection, self.name)
+    v = self.scope.get_variable(self.collection, self.name)
+    return meta.unbox(v) if self.unbox else v
 
   @value.setter
   def value(self, value: T):
     """Updates the value of this Variable."""
+    if self.unbox:
+      cur = self.scope.get_variable(self.collection, self.name)
+      cur_struct = tree_util.tree_structure(cur,
+                                            is_leaf=meta.is_axis_metadata)
+      value_struct = tree_util.tree_structure(value,
+                                              is_leaf=meta.is_axis_metadata)
+      has_meta = any(map(meta.is_axis_metadata,
+                         cur_struct.flatten_up_to(cur)))
+      if cur_struct == value_struct and has_meta:
+        value = meta.replace_boxed(cur, value)
+
     self.scope.put_variable(self.collection, self.name, value)
 
   def is_mutable(self) -> bool:
@@ -395,7 +412,7 @@ class Scope:
                mutable: CollectionFilter = False,
                parent: Optional['Scope'] = None,
                path: Iterable[str] = (),
-               flags: Optional[Dict] = None):
+               flags: Optional[Mapping] = None):
     """Initializes a Scope.
 
     Args:
@@ -503,6 +520,8 @@ class Scope:
   def reserve(self, name: str):
     """Reserves a name for a child Scope or Variable.
 
+    Throws an error if the name exists already.
+
     Args:
       name: the name to reserve.
     """
@@ -553,10 +572,10 @@ class Scope:
     rngs = {key: LazyRng.create(rng, name) for key, rng in self.rngs.items()}
     rng_key = (child_rng_token, name)
     if rng_key in self.rng_counters:
-      rng_counters = self.rng_counters.get(rng_key)
+      rng_counters = self.rng_counters.get(rng_key) # type: ignore
     else:
       rng_counters = {key: 0 for key in rngs}
-      self.rng_counters[rng_key] = rng_counters
+      self.rng_counters[rng_key] = rng_counters # type: ignore
     scope = Scope({},
                   name=name,
                   rngs=rngs,
@@ -619,20 +638,36 @@ class Scope:
   def _mutable_collection(self, col: str) -> MutableCollection:
     """Returns the collection `col` as a mutable object."""
     assert self.is_mutable_collection(col), f'Collection {col} is not mutable'
+
+    # The actual variable dict is stored in the root scope only, and subscopes
+    # hold references to subtrees relevant to them. This function ensures that
+    # the collections are created in the top-level Scope and we return the
+    # correct reference.
     if col not in self._variables:
-      if self.parent:
+      if not self.parent:
+        # If this is the top-level Scope, just add an empty collection.
+        self._variables[col] = {}
+      else:
+        assert self.name is not None  # Only top-level Scope have name None.
+        # Populate the parent collections recursively and obtain a reference to
+        # the direct parent (which, by transitivity, is be a reference to a
+        # dict in the root Scope).
         parent_col = self.parent._mutable_collection(col)  # pylint: disable=protected-access
         if self.name not in parent_col:
+          # If this Scope's name does not occur in the parent collection, add it
+          # to the parent scope (updating the parent's variable dict).
           parent_col[self.name] = {}
+        # Store a reference to the parent's scope collection for in this scope's
+        # variable dict.
         self._variables[col] = parent_col[self.name]
-      else:
-        self._variables[col] = {}
+
     return self._variables[col]
 
   def _collection(self, col: str) -> Collection:
     """Returns a collection of variables of collection `col`."""
     if col not in self._variables:
       if self.parent:
+        assert self.name is not None
         parent_col = self.parent._collection(col)  # pylint: disable=protected-access
         if self.name not in parent_col:
           return FrozenDict()
@@ -696,7 +731,9 @@ class Scope:
     if not self.is_mutable_collection(col):
       raise errors.ModifyScopeVariableError(col, name, self.path_text)
     variables = self._mutable_collection(col)
-    # Make sure reference sharing of child variable dictionaries isn't broken
+
+    # Make sure reference sharing of child variable dictionaries isn't broken.
+    # See https://github.com/google/flax/issues/2022 for more details.
     def put(target, key, val):
       if (key in target and isinstance(target[key], dict) and
           isinstance(val, Mapping)):
@@ -709,7 +746,7 @@ class Scope:
 
   def variable(self, col: str, name: str,  # pylint: disable=keyword-arg-before-vararg
                init_fn: Optional[Callable[..., T]] = None,
-               *init_args) -> Variable[T]:
+               *init_args, unbox: bool = True) -> Variable[T]:
     """Creates a variable if it doesn't exist yet in this scope and returns it.
 
     Args:
@@ -719,9 +756,11 @@ class Scope:
         arguments. If None, the variable must already be initialized otherwise
         an error is raised.
       *init_args: the arguments to evaluate init_fn on lazily.
+      unbox: If True, ``AxisMetadata`` instances are replaced by their unboxed
+        value, see ``flax.nn.meta.unbox`` (default: True).
 
     Returns:
-      The variable.
+      The variable.  Throws an error if the variable exists already.
     """
     self.reserve(name)
     if not self.has_variable(col, name):
@@ -731,9 +770,10 @@ class Scope:
         raise errors.ScopeVariableNotFoundError(name, col, self.path_text)
       init_value = init_fn(*init_args)
       self.put_variable(col, name, init_value)
-    return Variable(self, col, name)
+    return Variable(self, col, name, unbox=unbox)
 
-  def param(self, name: str, init_fn: Callable[..., T], *init_args) -> T:
+  def param(self, name: str, init_fn: Callable[..., T], *init_args,
+            unbox: bool = True) -> T:
     """Creates a parameter if it doesn't exist yet in this scope and returns it.
 
     If the parameter exists already, the existing value is simply returned.
@@ -743,9 +783,11 @@ class Scope:
       init_fn: a function taking a PRNGKey plus any other number of positional
         arguments.
       *init_args: the arguments to evaluate init_fn on lazily.
+      unbox: If True, ``AxisMetadata`` instances are replaced by their unboxed
+        value, see ``flax.nn.meta.unbox`` (default: True).
 
     Returns:
-      The parameters.
+      The parameters. Throws an error if the params exist already.
     """
     self.reserve(name)
     if self.has_variable('params', name):
@@ -774,7 +816,8 @@ class Scope:
         raise errors.ScopeParamNotFoundError(name, self.path_text)
       value = init_fn(self.make_rng('params'), *init_args)
       self.put_variable('params', name, value)
-
+    if unbox:
+      value = meta.unbox(value)
     return value
 
   def _populate_collections(self):
@@ -804,7 +847,7 @@ def _unfreeze_variables(variables, mutable):
 def bind(variables: VariableDict,
          rngs: Optional[RNGSequences] = None,
          mutable: CollectionFilter = False,
-         flags: Optional[Dict] = None):
+         flags: Optional[Mapping] = None):
   """Binds variables and rngs to a new ``Scope``.
 
   bind provides a ``Scope`` instance without transforming a function with
@@ -837,7 +880,7 @@ def bind(variables: VariableDict,
 
 def apply(fn: Callable[..., Any],
           mutable: CollectionFilter = False,
-          flags: Optional[Dict] = None) -> Callable[..., Any]:
+          flags: Optional[Mapping] = None) -> Callable[..., Any]:
   """Functionalize a `Scope` function.
 
   Args:
@@ -873,7 +916,7 @@ def apply(fn: Callable[..., Any],
 
 def init(fn: Callable[..., Any],
          mutable: CollectionFilter = True,
-         flags: Optional[Dict] = None) -> Callable[..., Any]:
+         flags: Optional[Mapping] = None) -> Callable[..., Any]:
   """Functionalize a `Scope` function for initialization.
 
   Args:
@@ -930,7 +973,7 @@ def _is_valid_variables(variables: VariableDict) -> bool:
 def _is_valid_rng(rng: Array):
   """Checks whether rng is a valid JAX PRNGKey, also handling custom prngs."""
   # New-style JAX KeyArrays have a base type.
-  if jax_config.jax_enable_custom_prng:
+  if jax_config.jax_enable_custom_prng: # type: ignore[attr-defined]
     if not isinstance(rng, jax.random.KeyArray):
       return False
   # Old-style JAX PRNGKeys are plain uint32 arrays.
