@@ -15,7 +15,7 @@
 """The Flax Module system."""
 
 
-# pylint: disable=g-multiple-import
+# pylint: disable=g-multiple-import,useless-import-alias
 # re-export commonly used modules and functions
 from .activation import (
   PReLU as PReLU,
@@ -41,6 +41,7 @@ from .activation import (
   soft_sign as soft_sign,
   softmax as softmax,
   softplus as softplus,
+  standardize as standardize,
   swish as swish,
   tanh as tanh
 )
@@ -57,7 +58,25 @@ from .combinators import Sequential as Sequential
 from ..core import (
   DenyList as DenyList,
   FrozenDict as FrozenDict,
-  broadcast as broadcast
+  broadcast as broadcast,
+  meta as meta,
+)
+from ..core.meta import (
+    Partitioned as Partitioned,
+    with_partitioning as with_partitioning,
+    get_partition_spec as get_partition_spec,
+    unbox as unbox,
+    PARTITION_NAME as PARTITION_NAME,
+)
+from .spmd import (
+    logical_axis_rules as logical_axis_rules,
+    set_logical_axis_rules as set_logical_axis_rules,
+    get_logical_axis_rules as get_logical_axis_rules,
+    logical_to_mesh_axes,
+    logical_to_mesh,
+    with_logical_constraint,
+    LogicallyPartitioned as LogicallyPartitioned,
+    with_logical_partitioning as with_logical_partitioning,
 )
 from .initializers import (
   ones as ones,
@@ -95,7 +114,7 @@ from .pooling import (
   pool as pool
 )
 from .recurrent import (
-  ConvLSTM as ConvLSTM,
+  ConvLSTMCell as ConvLSTMCell,
   GRUCell as GRUCell,
   LSTMCell as LSTMCell,
   OptimizedLSTMCell as OptimizedLSTMCell
@@ -116,6 +135,7 @@ from .transforms import (
   while_loop as while_loop,
   cond as cond,
   switch as switch,
+  add_metadata_axis,
 )
 from .summary import tabulate
 # pylint: enable=g-multiple-import
