@@ -14,7 +14,6 @@
 
 """Linear modules."""
 
-import abc
 import dataclasses
 from typing import (Any, Callable, Iterable, List, Optional, Sequence, Tuple,
                     Union)
@@ -27,9 +26,11 @@ from flax.linen.module import Module
 from flax.linen.dtypes import promote_dtype
 from jax import eval_shape
 from jax import lax
+from jax import random
 from jax import ShapedArray
 import jax.numpy as jnp
 import numpy as np
+import jax
 
 
 PRNGKey = Any
@@ -106,11 +107,11 @@ class DenseGeneral(Module):
     n_axis, n_features = len(axis), len(features)
 
     def kernel_init_wrap(rng, shape, dtype=jnp.float32):
-      size_batch_dims = np.prod(shape[:n_batch_dims], dtype=np.int32)
-      flat_shape = (np.prod(shape[n_batch_dims:n_axis + n_batch_dims]),
+      flat_shape = (np.prod(shape[:n_batch_dims]) *
+                    np.prod(shape[n_batch_dims:n_axis + n_batch_dims]),
                     np.prod(shape[-n_features:]),)
-      kernel = jnp.concatenate([self.kernel_init(rng, flat_shape, dtype)
-                                for _ in range(size_batch_dims)], axis=0)
+      flat_shape = jax.tree_map(int, flat_shape)
+      kernel = self.kernel_init(rng, flat_shape, dtype)
       return jnp.reshape(kernel, shape)
 
     batch_shape = tuple(inputs.shape[ax] for ax in batch_dims)
@@ -127,10 +128,10 @@ class DenseGeneral(Module):
 
     if self.use_bias:
       def bias_init_wrap(rng, shape, dtype=jnp.float32):
-        size_batch_dims = np.prod(shape[:n_batch_dims], dtype=np.int32)
-        flat_shape = (np.prod(shape[-n_features:]),)
-        bias = jnp.concatenate([self.bias_init(rng, flat_shape, dtype)
-                                for _ in range(size_batch_dims)], axis=0)
+        flat_shape = (np.prod(shape[:n_batch_dims]) *
+                      np.prod(shape[-n_features:]),)
+        flat_shape = jax.tree_map(int, flat_shape)
+        bias = self.bias_init(rng, flat_shape, dtype)
         return jnp.reshape(bias, shape)
 
       bias = self.param('bias', bias_init_wrap, batch_shape + features,
@@ -290,7 +291,7 @@ class _Conv(Module):
   bias_init: Callable[[PRNGKey, Shape, Dtype], Array] = zeros
 
   @property
-  def shared_weights(self) -> bool:
+  def shared_weights(self) -> bool:  # type: ignore
     """Defines whether weights are shared or not between different pixels.
 
     Returns:
@@ -580,6 +581,8 @@ class ConvTranspose(Module):
       for details.
     kernel_init: initializer for the convolutional kernel.
     bias_init: initializer for the bias.
+    transpose_kernel: if True flips spatial axes and swaps the input/output
+      channel axes of the kernel.
   """
   features: int
   kernel_size: Union[int, Tuple[int, ...]]
@@ -593,6 +596,7 @@ class ConvTranspose(Module):
   precision: PrecisionLike = None
   kernel_init: Callable[[PRNGKey, Shape, Dtype], Array] = default_kernel_init
   bias_init: Callable[[PRNGKey, Shape, Dtype], Array] = zeros
+  transpose_kernel: bool = False
 
   @compact
   def __call__(self, inputs: Array) -> Array:
@@ -635,7 +639,10 @@ class ConvTranspose(Module):
     strides = self.strides or (1,) * (inputs.ndim - 2)
 
     in_features = jnp.shape(inputs)[-1]
-    kernel_shape = kernel_size + (in_features, self.features)
+    if self.transpose_kernel:
+      kernel_shape = kernel_size + (self.features, in_features)
+    else:
+      kernel_shape = kernel_size + (in_features, self.features)
 
     if self.mask is not None and self.mask.shape != kernel_shape:
       raise ValueError('Mask needs to have the same shape as weights. '
@@ -666,6 +673,7 @@ class ConvTranspose(Module):
         strides,
         padding_lax,
         rhs_dilation=self.kernel_dilation,
+        transpose_kernel=self.transpose_kernel,
         precision=self.precision)
 
     if self.padding == 'CIRCULAR':
@@ -688,12 +696,20 @@ class ConvTranspose(Module):
           -(y_dim - x_dim) % (2 * x_dim)
           for y_dim, x_dim in zip(y.shape[1:-1], scaled_x_dims)
       ]
-      # Divide the padding equaly between left and right. The choice to put
-      # "+1" on the left (and not on the right) represents a convention for
-      # aligning even-sized kernels.
-      total_pad = [
-          ((size_diff + 1) // 2, size_diff // 2) for size_diff in size_diffs
-      ]
+      if self.transpose_kernel:
+        # If the kernel is transposed, the "+1" is put on the right to
+        # mirror the regular convolution. If the same kernel parameters are used
+        # as for Conv, this layer then computes the proper transpose convolution.
+        total_pad = [
+            (size_diff // 2, (size_diff + 1) // 2) for size_diff in size_diffs
+        ]
+      else:
+        # Divide the padding equally between left and right. The choice to put
+        # "+1" on the left (and not on the right) represents a convention for
+        # aligning even-sized kernels.
+        total_pad = [
+            ((size_diff + 1) // 2, size_diff // 2) for size_diff in size_diffs
+        ]
       y = jnp.pad(y, [(0, 0)] + total_pad + [(0, 0)])
       # Wrap the result periodically around each spatial dimension,
       # one by one.
