@@ -22,14 +22,14 @@ import re
 import threading
 import typing
 import weakref
-from typing import (Any, Callable, Dict, Iterable, List, NamedTuple, Mapping,
+from typing import (Any, Callable, Dict, Iterable, List, Sequence, NamedTuple, Mapping,
                     Optional, Set, Tuple, Type, TypeVar, Union, overload)
 
 import jax
 import numpy as np
 import jax.numpy as jnp
-from typing_extensions import \
-    dataclass_transform  # pytype: disable=not-supported-yet
+from typing_extensions import Protocol, \
+  dataclass_transform  # pytype: disable=not-supported-yet
 
 import flax
 from flax import (config, core, errors, serialization, traceback_util,
@@ -53,6 +53,7 @@ Array = Any    # pylint: disable=invalid-name
 
 T = TypeVar('T')
 K = TypeVar('K')
+M = TypeVar('M', bound='Module')
 _CallableT = TypeVar('_CallableT', bound=Callable)
 
 
@@ -96,8 +97,9 @@ def _module_repr(module: 'Module', num_spaces: int = 4):
   cls = type(module)
   cls_name = cls.__name__
   rep = ''
-  attributes = {k: v for k, v in cls.__annotations__.items()
-                if k not in ('parent', 'name')}
+
+  attributes = {f.name: f.type for f in dataclasses.fields(cls)
+                if f.name not in ('parent', 'name')}
   child_modules = {k: v for k, v in module._state.children.items()  # pytype: disable=attribute-error
                    if isinstance(v, Module)}
   if attributes:
@@ -380,12 +382,33 @@ def _get_local_method_names(cls: Any,
   """
   true_methods = set()
   for m in cls.__dict__:
-    if callable(cls.__dict__[m]) and not inspect.isclass(cls.__dict__[m]):
+    if callable(cls.__dict__[m]) and not inspect.isclass(cls.__dict__[m]):  # pytype: disable=not-supported-yet
       mtype = type(cls.__dict__[m])
       if mtype != staticmethod and mtype != classmethod:
         true_methods.add(m)
   return tuple(true_methods.difference(set(exclude)))
 
+def _get_local_descriptor_names(cls: Any,
+                                exclude: Iterable[str] = ()) -> Tuple[str, ...]:
+  """Gets descriptor names of a class.
+
+  Args:
+    cls: The class to get property names for.
+    exclude: Names to exclude from output.
+  Returns:
+    A list of property names.
+  """
+  true_properties = set()
+  for m, attr in cls.__dict__.items():
+    if not callable(attr) and (
+      hasattr(attr, '__get__') or hasattr(attr, '__set__') or
+      hasattr(attr, '__delete__')
+    ):
+      mtype = type(attr)
+      if mtype != staticmethod and mtype != classmethod:
+        true_properties.add(m)
+  return tuple(true_properties.difference(set(exclude)))
+
 
 def wrap_method_once(fun: Callable[..., Any]) -> Callable[..., Any]:
   """Manages Module state for a given user-defined method.
@@ -414,6 +437,20 @@ def wrap_method_once(fun: Callable[..., Any]) -> Callable[..., Any]:
   wrapped_module_method.method_handler_wrapped = True  # type: ignore[attr-defined]
   return wrapped_module_method
 
+def wrap_descriptor_once(descriptor) -> "DescriptorWrapper":
+  """Wraps a descriptor to give better error messages.
+
+  Args:
+    prop: User-defined Module attribute descriptor.
+  Returns:
+    Wrapped descriptor.
+  """
+  # Don't rewrap descriptors.
+  if isinstance(descriptor, DescriptorWrapper):
+    return descriptor
+
+  return create_descriptor_wrapper(descriptor)
+
 
 def _wrap_hash(hash_fn: Callable[..., Any]) -> Callable[..., Any]:
   """Wraps a hash function with some check for Flax Modules."""
@@ -528,9 +565,6 @@ tuple_init = lambda: ()
 capture_call_intermediates = lambda _, method_name: method_name == '__call__'
 
 
-_ParentType = Union[Type['Module'], Type[Scope], Type[_Sentinel], None]
-
-
 class ParentDescriptor:
   """Wraps parent module references in weak refs.
 
@@ -555,6 +589,55 @@ class ParentDescriptor:
     object.__setattr__(obj, "_parent_ref", maybe_weak)
 
 
+class Descriptor(Protocol):
+  __isabstractmethod__: bool
+  def __get__(self, obj, objtype=None) -> Any: ...
+  def __set__(self, obj, value) -> None: ...
+  def __delete__(self, obj) -> None: ...
+  def __set_name__(self, owner, name) -> None: ...
+
+class DescriptorWrapper:
+  pass
+
+def create_descriptor_wrapper(descriptor: Descriptor):
+  """Creates a descriptor wrapper that calls a get_fn on the descriptor."""
+
+  class _DescriptorWrapper(DescriptorWrapper):
+    """A descriptor that can wrap any descriptor"""
+
+    if hasattr(descriptor, '__isabstractmethod__'):
+      __isabstractmethod__ = descriptor.__isabstractmethod__
+
+    def __init__(self, wrapped: Descriptor):
+      self.wrapped = wrapped
+
+    # conditionally define descriptor methods
+    if hasattr(descriptor, '__get__'):
+      def __get__(self, *args, **kwargs):
+        # here we will catch internal AttributeError and re-raise it as a
+        # more informative and correct error message.
+        try:
+          return self.wrapped.__get__(*args, **kwargs)
+        except AttributeError as e:
+          raise errors.DescriptorAttributeError() from e
+
+    if hasattr(descriptor, '__set__'):
+      def __set__(self, *args, **kwargs):
+        return self.wrapped.__set__(*args, **kwargs)
+
+    if hasattr(descriptor, '__delete__'):
+      def __delete__(self, *args, **kwargs):
+        return self.wrapped.__delete__(*args, **kwargs)
+
+    if hasattr(descriptor, '__set_name__'):
+      def __set_name__(self, *args, **kwargs):
+        self.wrapped.__set_name__(*args, **kwargs)
+
+    def __getattr__(self, name):
+      return getattr(self.wrapped, name)
+
+  return _DescriptorWrapper(descriptor)
+
 # Base Module definition.
 # -----------------------------------------------------------------------------
 
@@ -616,13 +699,13 @@ class Module:
     # We wrap user-defined methods including setup and __call__ to enforce
     # a number of different checks and to provide clear error messages.
     cls._verify_single_or_no_compact()
-    cls._wrap_module_methods()
+    cls._wrap_module_attributes()
     # Set empty class defaults.
-    cls._state = _uninitialized_module_internal_state
-    cls.scope: Optional[Scope] = None
+    cls._state = _uninitialized_module_internal_state # type: ignore[attr-defined]
+    cls.scope: Optional[Scope] = None # type: ignore
     # Handles weak referencing of parent Modules to prevent reference cycles.
-    cls._parent_ref = None
-    cls.parent = ParentDescriptor()
+    cls._parent_ref = None # type: ignore[attr-defined]
+    cls.parent = ParentDescriptor() # type: ignore[attr-defined]
 
   @classmethod
   def _customized_dataclass_transform(cls):
@@ -676,16 +759,29 @@ class Module:
       raise errors.MultipleMethodsCompactError()
 
   @classmethod
-  def _wrap_module_methods(cls):
-    """Wraps user-defined non-inherited methods with state management functions."""
-    exclusions = ([f.name for f in dataclasses.fields(cls)] +
+  def _wrap_module_attributes(cls):
+    """Wraps user-defined non-inherited methods and descriptors with state
+    management functions.
+    """
+    # wrap methods
+    method_exclusions = ([f.name for f in dataclasses.fields(cls)] +
                   ['__eq__', '__repr__', '__init__', '__hash__',
                    '__post_init__'])
-    for key in _get_local_method_names(cls, exclude=exclusions):
+    for key in _get_local_method_names(cls, exclude=method_exclusions):
       method = getattr(cls, key)
       if hasattr(method, 'nowrap'):
         continue
       setattr(cls, key, wrap_method_once(method))
+
+    # wrap descriptors
+    descriptor_exclusions = ([f.name for f in dataclasses.fields(cls)] +
+                             ['parent', '__dict__'])
+    for key in _get_local_descriptor_names(cls, descriptor_exclusions):
+      # don't use getattr here, since it will call the descriptor
+      descriptor = cls.__dict__[key]
+      if hasattr(descriptor, 'nowrap'):
+        continue
+      setattr(cls, key, wrap_descriptor_once(descriptor))
     return cls
 
   def _call_wrapped_method(self, fun, args, kwargs):
@@ -815,7 +911,7 @@ class Module:
   def __dir__(self) -> List[str]:
     """Call setup() before listing attributes."""
     self._try_setup()
-    return object.__dir__(self)  # pytype: disable=attribute-error
+    return object.__dir__(self)  # type: ignore
 
   def __post_init__(self) -> None:
     # DO NOT REMOVE - Marker for internal logging.
@@ -842,6 +938,7 @@ class Module:
       # When initializing an unnamed Module inside setup()
       # initialization is deferred until attachment by __setattr__
       # i.e. self.mymodule = MyModule(...)
+      self.name: Optional[str]
       if self.parent._state.in_setup and self.name is None:  # pytype: disable=attribute-error
         return
       if not self.parent._initialization_allowed:
@@ -976,7 +1073,7 @@ class Module:
 
   def _name_taken(self,
                   name: str,
-                  module: 'Module' = None,
+                  module: Optional['Module'] = None,
                   reuse_scopes: bool = False) -> bool:
     if name in _all_names_on_object(self):
       val = getattr(self, name, None)
@@ -994,9 +1091,9 @@ class Module:
   def _initialization_allowed(self):
     return self._state.in_setup or self._state.in_compact_method
 
-  def clone(self, *,
+  def clone(self: M, *,
             parent: Optional[Union[Scope, 'Module']] = None,
-            **updates) -> 'Module':
+            **updates) -> M:
     """Creates a clone of this Module, with optionally updated arguments.
 
     Args:
@@ -1012,7 +1109,8 @@ class Module:
 
   def variable(self, col: str, name: str,
                init_fn: Optional[Callable[..., Any]] = None,
-               *init_args) -> Variable:
+               *init_args,
+               unbox: bool = True) -> Variable:
     """Declares and returns a variable in this Module.
 
     See :mod:`flax.core.variables` for more information. See also :meth:`param`
@@ -1037,6 +1135,8 @@ class Module:
         this variable is used in this module. If None, the variable must
         already be initialized otherwise an error is raised.
       *init_args: The arguments to pass to init_fn.
+      unbox: If True, ``AxisMetadata`` instances are replaced by their unboxed
+        value, see ``flax.nn.meta.unbox`` (default: True).
 
     Returns:
       A :class:`flax.core.variables.Variable` that can be read or set via
@@ -1048,11 +1148,12 @@ class Module:
           'wrapped in `@compact`')
     if self._name_taken(name):
       raise errors.NameInUseError('variable', name, self.__class__.__name__)
-    v = self.scope.variable(col, name, init_fn, *init_args)
+    v = self.scope.variable(col, name, init_fn, *init_args, unbox=unbox)
     self._state.children[name] = col
     return v
 
-  def param(self, name: str, init_fn: Callable[..., T], *init_args) -> T:
+  def param(self, name: str, init_fn: Callable[..., T], *init_args,
+            unbox: bool = True) -> T:
     """Declares and returns a parameter in this Module.
 
     Parameters are read-only variables in the collection named "params". See
@@ -1074,9 +1175,12 @@ class Module:
         of this variable. This function will only be called the first time
         this parameter is used in this module.
       *init_args: The arguments to pass to init_fn.
+      unbox: If True, ``AxisMetadata`` instances are replaced by their unboxed
+        value, see ``flax.nn.meta.unbox`` (default: True).
 
     Returns:
-      The value of the initialized parameter.
+      The value of the initialized parameter. Throws an error if the parameter
+      exists already.
     """
     if not self._initialization_allowed:
       raise ValueError(
@@ -1084,7 +1188,7 @@ class Module:
           'wrapped in `@compact`')
     if self._name_taken(name):
       raise errors.NameInUseError('param', name, self.__class__.__name__)
-    v = self.scope.param(name, init_fn, *init_args)
+    v = self.scope.param(name, init_fn, *init_args, unbox=unbox)
     self._state.children[name] = 'params'
     return v
 
@@ -1159,23 +1263,23 @@ class Module:
       raise errors.IncorrectPostInitOverrideError()
 
   @traceback_util.api_boundary
-  def bind(self,
+  def bind(self: M,
            variables: VariableDict,
            *args,
            rngs: Optional[RNGSequences] = None,
-           mutable: CollectionFilter = False):
+           mutable: CollectionFilter = False) -> M:
     """Creates an interactive Module instance by binding variables and RNGs.
 
     ``bind`` provides an "interactive" instance of a Module directly without
-    transforming a function with ``apply``. This is particularly useful for debugging
-    and interactive use cases like notebooks where a function would limit the ability
-    to split up code into different cells.
+    transforming a function with ``apply``. This is particularly useful for
+    debugging zand interactive use cases like notebooks where a function would
+    limit the ability to split up code into different cells.
 
-    Once the variables (and optionally RNGs) are bound to a ``Module`` it becomes a
-    stateful object. Note that idiomatic JAX is functional and therefore an interactive
-    instance does not mix well with vanilla JAX APIs. ``bind()`` should only be used
-    for interactive experimentation, and in all other cases we strongly encourage users
-    to use ``apply()`` instead.
+    Once the variables (and optionally RNGs) are bound to a ``Module`` it
+    becomes a stateful object. Note that idiomatic JAX is functional and
+    therefore an interactive instance does not mix well with vanilla JAX APIs.
+    ``bind()`` should only be used for interactive experimentation, and in all
+    other cases we strongly encourage users to use ``apply()`` instead.
 
     Example::
 
@@ -1198,7 +1302,6 @@ class Module:
       z = model.encoder(x)
       x_reconstructed = model.decoder(z)
 
-
     Args:
       variables: A dictionary containing variables keyed by variable
         collections. See :mod:`flax.core.variables` for more details
@@ -1210,6 +1313,7 @@ class Module:
           ``bool``: all/no collections are mutable.
           ``str``: The name of a single mutable collection.
           ``list``: A list of names of mutable collections.
+
     Returns:
       A copy of this instance with bound variables and RNGs.
     """
@@ -1219,12 +1323,48 @@ class Module:
     scope = core.bind(variables, rngs=rngs, mutable=mutable)
     return self.clone(parent=scope)
 
+  def unbind(self: M) -> Tuple[M, VariableDict]:
+    """Returns an unbound copy of a Module and its variables.
+
+    ``unbind`` helps create a stateless version of a bound Module.
+
+    An example of a common use case: to extract a sub-Module defined inside
+    ``setup()`` and its corresponding variables: 1) temporarily ``bind`` the parent
+    Module; and then 2) ``unbind`` the desired sub-Module. (Recall that ``setup()``
+    is only called when the Module is bound.)::
+
+      class AutoEncoder(nn.Module):
+        def setup(self):
+          self.encoder = Encoder()
+          self.decoder = Decoder()
+
+        def __call__(self, x):
+          return self.decoder(self.encoder(x))
+
+      module = AutoEncoder()
+      variables = module.init(jax.random.PRNGKey(0), jnp.ones((1, 784)))
+      ...
+      # Extract the Encoder sub-Module and its variables
+      encoder, encoder_vars = module.bind(variables).encoder.unbind()
+
+    Returns:
+      A tuple with an unbound copy of this Module and its variables.
+    """
+    Module._module_checks(self)
+
+    if self.scope is None:
+      raise errors.CallUnbindOnUnboundModuleError()
+
+    variables = self.variables
+    module = self.clone()
+    return module, variables
+
   @traceback_util.api_boundary
   def apply(self,
             variables: VariableDict,
             *args,
             rngs: Optional[RNGSequences] = None,
-            method: Optional[Callable[..., Any]] = None,
+            method: Union[Callable[..., Any], str, None] = None,
             mutable: CollectionFilter = False,
             capture_intermediates: Union[bool, Callable[['Module', str], bool]] = False,
             **kwargs) -> Union[Any, Tuple[Any, FrozenVariableDict]]:
@@ -1243,6 +1383,11 @@ class Module:
 
       encoded = model.apply({'params': params}, x, method=model.encode)
 
+    You can also pass a string to a callable attribute of the module. For
+    example, the previous can be written as::
+
+      encoded = model.apply({'params': params}, x, method='encode')
+
     Note ``method`` can also be a function that is not defined in
     ``Transformer``. In that case, the function should have at least one
     argument representing an instance of the Module class::
@@ -1262,7 +1407,8 @@ class Module:
         The "params" PRNG sequence is used to initialize parameters.
       method: A function to call apply on. This is generally a function in the
         module. If provided, applies this method. If not provided, applies the
-        ``__call__`` method of the module.
+        ``__call__`` method of the module. A string can also be provided to
+        specify a method by name.
       mutable: Can be bool, str, or list. Specifies which collections should be
                treated as mutable: ``bool``: all/no collections are mutable.
                ``str``: The name of a single mutable collection. ``list``: A
@@ -1281,7 +1427,13 @@ class Module:
     """
     Module._module_checks(self)
 
-    if method is None:
+    if isinstance(method, str):
+      attribute_name = method
+      method = getattr(self, attribute_name)
+      if not callable(method):
+        class_name = type(self).__name__
+        raise TypeError(f'\'{class_name}.{attribute_name}\' must be a callable, got {type(method)}.')
+    elif method is None:
       method = self.__call__
     method = _get_unbound_fn(method)
     return apply(
@@ -1294,7 +1446,7 @@ class Module:
   def init_with_output(self,
                        rngs: Union[PRNGKey, RNGSequences],
                        *args,
-                       method: Optional[Callable[..., Any]] = None,
+                       method: Union[Callable[..., Any], str, None] = None,
                        mutable: CollectionFilter = DenyList('intermediates'),
                        capture_intermediates: Union[bool, Callable[['Module', str], bool]] = False,
                        **kwargs) -> Tuple[Any, FrozenVariableDict]:
@@ -1304,7 +1456,8 @@ class Module:
       rngs: The rngs for the variable collections.
       *args: Named arguments passed to the init function.
       method: An optional method. If provided, applies this method. If not
-        provided, applies the ``__call__`` method.
+        provided, applies the ``__call__`` method. A string can also be'
+        provided to specify a method by name.
       mutable: Can be bool, str, or list. Specifies which collections should be
         treated as mutable: ``bool``: all/no collections are mutable.
         ``str``: The name of a single mutable collection. ``list``: A
@@ -1329,7 +1482,14 @@ class Module:
             'RNGs should be of shape (2,) or KeyArray in Module '
             f'{self.__class__.__name__}, but rngs are: {rngs}')
       rngs = {'params': rngs}
-    if method is None:
+
+    if isinstance(method, str):
+      attribute_name = method
+      method = getattr(self, attribute_name)
+      if not callable(method):
+        class_name = type(self).__name__
+        raise TypeError(f'\'{class_name}.{attribute_name}\' must be a callable, got {type(method)}.')
+    elif method is None:
       method = self.__call__
     method = _get_unbound_fn(method)
     return init_with_output(
@@ -1343,24 +1503,73 @@ class Module:
   def init(self,
            rngs: Union[PRNGKey, RNGSequences],
            *args,
-           method: Optional[Callable[..., Any]] = None,
+           method: Union[Callable[..., Any], str, None] = None,
            mutable: CollectionFilter = DenyList('intermediates'),
            capture_intermediates: Union[bool, Callable[['Module', str], bool]] = False,
            **kwargs) -> FrozenVariableDict:
     """Initializes a module method with variables and returns modified variables.
 
+    ``init`` takes as first argument either a single ``PRNGKey``, or a dictionary mapping variable collections names to their ``PRNGKeys``, and will call ``method`` (which is the module's ``__call__`` function by default) passing ``*args`` and ``**kwargs``, and returns
+    a dictionary of initialized variables.
+
+    Example::
+
+      >>> import flax.linen as nn
+      >>> import jax.numpy as jnp
+      >>> import jax
+      ...
+      >>> class Foo(nn.Module):
+      ...   @nn.compact
+      ...   def __call__(self, x, train):
+      ...     x = nn.Dense(16)(x)
+      ...     x = nn.BatchNorm(use_running_average=not train)(x)
+      ...     x = nn.relu(x)
+      ...     return nn.Dense(1)(x)
+      ...
+      >>> module = Foo()
+      >>> key = jax.random.PRNGKey(0)
+      >>> variables = module.init(key, jnp.empty((1, 7)), train=False)
+
+    If you pass a single ``PRNGKey``, Flax will use it to feed the ``'params'`` RNG stream.
+    If you want to use a different RNG stream or need to use multiple streams, you must pass a
+    dictionary mapping each RNG stream name to its corresponding ``PRNGKey`` to ``init``.
+
+    Example::
+
+      >>> class Foo(nn.Module):
+      ...   @nn.compact
+      ...   def __call__(self, x, train):
+      ...     x = nn.Dense(16)(x)
+      ...     x = nn.BatchNorm(use_running_average=not train)(x)
+      ...     x = nn.relu(x)
+      ...
+      ...     # Add gaussian noise
+      ...     noise_key = self.make_rng('noise')
+      ...     x = x + jax.random.normal(noise_key, x.shape)
+      ...
+      ...     return nn.Dense(1)(x)
+      ...
+      >>> module = Foo()
+      >>> rngs = {'params': jax.random.PRNGKey(0), 'noise': jax.random.PRNGKey(1)}
+      >>> variables = module.init(rngs, jnp.empty((1, 7)), train=False)
+
     Jitting `init` initializes a model lazily using only the shapes of the
     provided arguments, and avoids computing the forward pass with actual
     values. Example::
 
-      jit_init = jax.jit(SomeModule(...).init)
-      jit_init(rng, jnp.ones(input_shape, jnp.float32))
+      >>> module = nn.Dense(1)
+      >>> init_jit = jax.jit(module.init)
+      >>> variables = init_jit(jax.random.PRNGKey(0), jnp.empty((1, 7)))
+
+    ``init`` is a light wrapper over ``apply``, so other ``apply`` arguments like
+    ``method``, ``mutable``, and ``capture_intermediates`` are also available.
 
     Args:
       rngs: The rngs for the variable collections.
       *args: Named arguments passed to the init function.
       method: An optional method. If provided, applies this method. If not
-        provided, applies the ``__call__`` method.
+        provided, applies the ``__call__`` method. A string can also be
+        provided to specify a method by name.
       mutable: Can be bool, str, or list. Specifies which collections should be
         treated as mutable: ``bool``: all/no collections are mutable.
         ``str``: The name of a single mutable collection. ``list``: A
@@ -1394,7 +1603,7 @@ class Module:
       raise ValueError("Can't access variables on unbound modules")
     return self.scope.variables()
 
-  def get_variable(self, col: str, name: str, default: T = None) -> T:
+  def get_variable(self, col: str, name: str, default: Optional[T] = None) -> T:
     """Retrieves the value of a Variable.
 
     Args:
@@ -1412,19 +1621,16 @@ class Module:
     return self.scope.get_variable(col, name, default)
 
   def put_variable(self, col: str, name: str, value: Any):
-    """Sets the value of a Variable.
+    """Updates the value of the given variable if it is mutable, or an error otherwise.
 
     Args:
       col: the variable collection.
       name: the name of the variable.
       value: the new value of the variable.
-
-    Returns:
-
     """
     if self.scope is None:
       raise ValueError("Can't access variables on unbound modules")
-    return self.scope.put_variable(col, name, value)
+    self.scope.put_variable(col, name, value)
 
   @overload
   def sow(self, col: str, name: str, value: Any) -> bool:
@@ -1433,12 +1639,12 @@ class Module:
   @overload
   def sow(self, col: str, name: str, value: T,
           reduce_fn: Callable[[K, T], K] = tuple_reduce,
-          init_fn: Callable[[], K] = tuple_init) -> bool:
+          init_fn: Callable[[], K] = tuple_init) -> bool: # type: ignore
     ...
 
   def sow(self, col: str, name: str, value: T,
           reduce_fn: Callable[[K, T], K] = tuple_reduce,
-          init_fn: Callable[[], K] = tuple_init) -> bool:
+          init_fn: Callable[[], K] = tuple_init) -> bool: # type: ignore
     """Stores a value in a collection.
 
     Collections can be used to collect intermediate values without
@@ -1554,8 +1760,22 @@ class Module:
       print(intm_grads['dense3']) # ==> [[-1.456924   -0.44332537  0.02422847]
                                   #      [-1.456924   -0.44332537  0.02422847]]
 
+    If perturbations are not passed to `apply`, `perturb` behaves like a no-op
+    so you can easily disable the behavior when not needed::
+
+      model.apply({'params': params, 'perturbations': perturbations}, x) # works as expected
+      model.apply({'params': params}, x) # behaves like a no-op
+
     """
-    value += self.variable(collection, name, lambda: jnp.zeros_like(value)).value
+    def _root_has_collection():
+      """Returns True if the root scope has the collection."""
+      assert self.scope is not None
+      return collection in self.scope.root._variables
+    # we will only add the perturbation variable if the collection is mutable
+    # (e.g. during `init`) or if the collection was passed to `apply` (contained in
+    # the root scope).
+    if self.is_mutable_collection(collection) or _root_has_collection():
+      value += self.variable(collection, name, lambda: jnp.zeros_like(value)).value # type: ignore
     return value
 
   def tabulate(
@@ -1574,6 +1794,10 @@ class Module:
     the Module in a table. `tabulate` uses `jax.eval_shape` to run the forward
     computation without consuming any FLOPs or allocating memory.
 
+    Additional arguments can be passed into the `console_kwargs` argument, for example,
+    `{'width': 120}`. For a full list of `console_kwargs` arguments, see:
+    https://rich.readthedocs.io/en/stable/reference/console.html#rich.console.Console
+
     Example::
 
       import jax
@@ -1650,11 +1874,14 @@ class Module:
     return tabulate_fn(*args, **kwargs)
 
 
-def merge_param(name: str, a: Optional[T], b: Optional[T]) -> T:
-  """Merges construction and call time argument.
+_ParentType = Union[Type[Module], Type[Scope], Type[_Sentinel], None]
 
-  This is a utility for supporting the pattern where a Module hyper parameter
-  can be passed to ``__init__`` or ``__call__``.
+def merge_param(name: str, a: Optional[T], b: Optional[T]) -> T:
+  """Merges construction- and call-time argument.
+
+  This is a utility for supporting a pattern where a Module hyperparameter
+  can be passed either to ``__init__`` or ``__call__``, and the value that is
+  not `None` will be used.
 
   Example::
 
@@ -1664,7 +1891,8 @@ def merge_param(name: str, a: Optional[T], b: Optional[T]) -> T:
       def __call__(self, train: Optional[bool] = None):
         train = nn.merge_param('train', self.train, train)
 
-  An error is thrown when both arguments are `None` or both values are not `None`.
+  An error is thrown when both arguments are `None` or both values are not
+  `None`.
 
   Args:
     name: the name of the parameter. Used for error messages.
