@@ -250,6 +250,8 @@ class OptimizedLSTMCell(RNNCellBase):
       inputs, kernel, bias = promote_dtype(inputs, kernel, bias, dtype=self.dtype)
       y = jnp.dot(inputs, kernel)
       if use_bias:
+        # This assert is here since mypy can't infer that bias cannot be None
+        assert bias is not None
         y += jnp.reshape(bias, (1,) * (y.ndim - 1) + (-1,))
 
       # Split the result back into individual (i, f, g, o) outputs.
@@ -344,7 +346,7 @@ class GRUCell(RNNCellBase):
     """Gated recurrent unit (GRU) cell.
 
     Args:
-      carry: the hidden state of the LSTM cell,
+      carry: the hidden state of the GRU cell,
         initialized using `GRUCell.initialize_carry`.
       inputs: an ndarray with the input for the current time step.
         All dimensions except the final are considered batch dimensions.
@@ -393,7 +395,7 @@ class GRUCell(RNNCellBase):
     return init_fn(rng, mem_shape)
 
 
-class ConvLSTM(RNNCellBase):
+class ConvLSTMCell(RNNCellBase):
   r"""A convolutional LSTM cell.
 
   The implementation is based on xingjian2015convolutional.
