@@ -137,6 +137,10 @@ def tabulate(
   `tabulate` uses `jax.eval_shape` under the hood to run the forward computation without
   consuming any FLOPs or allocating memory.
 
+  Additional arguments can be passed into the `console_kwargs` argument, for example,
+  `{'width': 120}`. For a full list of `console_kwargs` arguments, see:
+  https://rich.readthedocs.io/en/stable/reference/console.html#rich.console.Console
+
   Example::
 
     import jax
@@ -277,7 +281,7 @@ def _get_module_variables(
   (module_variables, submodule_variables) tuple for that path. _get_module_variables
   uses the `all_paths` set to determine if a variable belongs to a submodule or not."""
   module_variables = _get_path_variables(path, variables)
-  submodule_variables = {collection: {} for collection in module_variables}
+  submodule_variables: Any = {collection: {} for collection in module_variables}
   all_keys = set(key for collection in module_variables.values() for key in collection)
 
   for key in all_keys:
