@@ -32,6 +32,7 @@ from flax import errors
 from flax import struct
 from flax import traceback_util
 from flax.core import lift
+from flax.core import meta
 from flax.core import Scope
 from flax.core.frozen_dict import FrozenDict
 from flax.linen import module as linen_module
@@ -44,7 +45,8 @@ import jax
 
 traceback_util.register_exclusion(__file__)
 
-# pylint: disable=protected-access
+# pylint: disable=protected-access,dangerous-default-value
+
 
 # Utils
 # -----------------------------------------------------------------------------
@@ -63,6 +65,7 @@ class VariablePlaceholder:
   """Used to mark Variables in a JAX-compatible way when lifting arguments."""
   collection: str = struct.field(pytree_node=False)
   name: str = struct.field(pytree_node=False)
+  unbox: bool = struct.field(pytree_node=False)
   id: int = struct.field(pytree_node=False)
 
 
@@ -126,7 +129,7 @@ def get_module_scopes(module, args=None, kwargs=None):
     nonlocal scopes
     if isinstance(x, Variable) and isinstance(x.scope, Scope):
       scopes.append(x.scope)
-      return VariablePlaceholder(x.collection, x.name, x._id)
+      return VariablePlaceholder(x.collection, x.name, x.unbox, x._id)
     elif isinstance(x, Module) and isinstance(x.scope, Scope):
       x._try_setup(shallow=True)
       scopes.append(x.scope)
@@ -197,7 +200,10 @@ def set_module_scopes(module, args, kwargs, scopes):
   def set_arg_scope(x):
     nonlocal idx
     if isinstance(x, VariablePlaceholder):
-      new_x = Variable(scope=scopes[idx], collection=x.collection, name=x.name)
+      new_x = Variable(scope=scopes[idx],
+                       collection=x.collection,
+                       name=x.name,
+                       unbox=x.unbox)
       idx += 1
       return new_x
     elif isinstance(x, InstancePlaceholder):
@@ -224,7 +230,8 @@ def set_module_scopes(module, args, kwargs, scopes):
       elif isinstance(x, Variable) and isinstance(x.scope, Scope):
         new_x = Variable(scope=scopes[idx],
                          collection=x.collection,
-                         name=x.name)
+                         name=x.name,
+                         unbox=x.unbox)
         idx += 1
         return new_x
       else:
@@ -438,6 +445,7 @@ def vmap(target: Target,
          axis_size: Optional[int] = None,
          axis_name: Optional[str] = None,
          spmd_axis_name: Optional[str] = None,
+         metadata_params: Mapping[Any, Any] = {},
          methods=None) -> Target:
   """A lifted version of ``jax.vmap``.
 
@@ -491,6 +499,8 @@ def vmap(target: Target,
     spmd_axis_name: Axis name added to any pjit sharding constraints appearing
       in `fn`. See also
       https://github.com/google/flax/blob/main/flax/linen/partitioning.py.
+    metadata_params: arguments dict passed to AxisMetadata instances in the
+      variable tree.
 
   Returns:
     A batched/vectorized version of ``target``, with the same arguments but with
@@ -507,6 +517,7 @@ def vmap(target: Target,
       out_axes=out_axes,
       axis_size=axis_size,
       axis_name=axis_name,
+      metadata_params=metadata_params,
       spmd_axis_name=spmd_axis_name)
 
 
@@ -698,6 +709,7 @@ def scan(target: Target,
          reverse: bool = False,
          unroll: int = 1,
          data_transform: Optional[Callable[..., Any]] = None,
+         metadata_params: Mapping[Any, Any] = {},
          methods=None) -> Target:
   """A lifted version of ``jax.lax.scan``.
 
@@ -794,6 +806,8 @@ def scan(target: Target,
     data_transform: optional function to transform raw functional-core variable
       and rng groups inside lifted scan body_fn, intended for inline SPMD
       annotations.
+    metadata_params: arguments dict passed to AxisMetadata instances in the
+      variable tree.
     methods: If `target` is a `Module`, the methods of `Module` to scan over.
 
   Returns:
@@ -812,6 +826,7 @@ def scan(target: Target,
       reverse=reverse,
       unroll=unroll,
       data_transform=data_transform,
+      metadata_params=metadata_params,
       methods=methods)
 
 
@@ -1339,3 +1354,42 @@ def named_call(class_fn, force=True):
     full_name = _derive_profiling_name(self, class_fn)
     return jax.named_call(class_fn, name=full_name)(self, *args, **kwargs)
   return wrapped_fn
+
+
+def add_metadata_axis(
+    target: Target,
+    variable_axes: Mapping[lift.CollectionFilter,
+                          lift.InOutAxis] = FrozenDict(),
+    metadata_params: Dict[Any, Any] = {}) -> Target:
+  """A helper to manipulate boxed axis metadata.
+
+  This is a helper to manipulate the *metadata* in boxed variables, similar
+  to how lifted ``vmap`` and ``scan`` will handle the introduction and stripping
+  of the new metadata axis across a transform boundary.
+
+  Args:
+    target: a ``Module`` or a function taking a ``Module``
+      as its first argument.
+    variable_axes: the variable collections whose axis metadata is being
+      transformed. Use `None` to indicate a broadcasted collection or an integer
+      to specify an axis index for an introduced axis.
+    methods: If `target` is a `Module`, the methods of `Module` to vmap over.
+    metadata_params: arguments dict passed to AxisMetadata instances in the
+      variable tree.
+  Returns:
+    A transformed version of ``target`` that performs a transform of the
+    axis metadata on its variables.
+  """
+  def add_fn(axis):
+    return lambda x: meta.add_axis(x, axis, metadata_params)
+  def remove_fn(axis):
+    return lambda x: meta.remove_axis(x, axis, metadata_params)
+  for col_name, axis in variable_axes.items():
+    target = map_variables(
+        target,
+        col_name,
+        trans_in_fn=remove_fn(axis),
+        trans_out_fn=add_fn(axis),
+        mutable=True,
+    )
+  return target
