@@ -24,22 +24,26 @@ import functools
 import os
 import pathlib
 import re
+import time
 from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
 from absl import logging
+from flax import config
+from flax import io
 from flax import core
 from flax import errors
 from flax import serialization
 from flax import traverse_util
 import jax
+from jax import monitoring
 from jax import process_index
 from jax import sharding
 from jax.experimental.global_device_array import GlobalDeviceArray
 from jax.experimental.multihost_utils import sync_global_devices
-from tensorflow import errors as tf_errors
-from tensorflow.io import gfile  # pytype: disable=import-error
-
+import orbax.checkpoint as orbax
 
+_READ_CHECKPOINT_EVENT: str = '/jax/checkpoint/read/durations_sec'
+_WRITE_CHECKPOINT_EVENT: str = '/jax/checkpoint/write/durations_sec'
 _IMPORT_GDAM_SUCCESSFUL = False
 try:
   from jax.experimental.gda_serialization.serialization import get_tensorstore_spec
@@ -76,14 +80,14 @@ MP_ARRAY_PH = '//GDAPlaceholder:'
 # We need this for GCS because GCS's directory move is not atomic.
 COMMIT_SUCCESS_FILE = 'commit_success.txt'
 
+# Orbax main checkpoint file name.
+ORBAX_CKPT_FILENAME = 'checkpoint'
+
 PyTree = Any
 
 # TODO(flax-dev): Remove this once flax is using the latest jax release
 # containing jax.Array attribute.
-if hasattr(jax, 'Array'):
-  MultiprocessArrayType = Union[GlobalDeviceArray, jax.Array]
-else:
-  MultiprocessArrayType = Union[GlobalDeviceArray, Any]
+MultiprocessArrayType = Union[GlobalDeviceArray, Any]
 
 
 def _checkpoint_path(ckpt_dir: str,
@@ -101,10 +105,17 @@ def _checkpoint_path_step(path: str) -> Optional[float]:
 
 def _allowempty_listdir(path: str):
   try:
-    return gfile.listdir(path)
-  except tf_errors.NotFoundError:
+    return io.listdir(path)
+  except io.NotFoundError:
     return []
 
+def _safe_remove(path: str):
+  """Identify whether a path is a dir or list and choose the correct remove method."""
+  if io.isdir(path):
+    io.rmtree(path)
+  else:
+    io.remove(path)
+
 class AsyncManager():
   """A simple object to track async checkpointing.
 
@@ -133,7 +144,7 @@ class AsyncManager():
       task: The callable to be executed asynchrously.
     """
     self.wait_previous_save()
-    self.save_future = self.executor.submit(task)
+    self.save_future = self.executor.submit(task) # type: ignore
 
 
 def _use_multiprocess_serialization(value: Any) -> bool:
@@ -174,17 +185,18 @@ def _make_mpa_dirs(mpa_targets: List[Tuple[MultiprocessArrayType, str]],
   mpa_tmp_path = tmp_path + MP_ARRAY_POSTFIX
   # Clean up the previous MPA dir, in case some leftover from last preemption
   # lingers.
-  if gfile.exists(mpa_tmp_path):
+  if io.exists(mpa_tmp_path):
     logging.info('Removing outdated MPA temporary files at %s', mpa_tmp_path)
-    gfile.rmtree(mpa_tmp_path)
+    io.rmtree(mpa_tmp_path)
   _, mpa_subpaths = zip(*mpa_targets)
   for subpath in mpa_subpaths:
-    gfile.makedirs(os.path.join(mpa_tmp_path, subpath))
+    io.makedirs(os.path.join(mpa_tmp_path, subpath))
 
 
 def _save_mpas(gda_manager, mpa_targets: List[Tuple[MultiprocessArrayType, str]],
                tmp_path: str, final_path: str, base_path: str, keep: int,
                overwrite: bool, keep_every_n_steps: Optional[int],
+               ckpt_start_time: float,
                async_manager: Optional[AsyncManager] = None):
   """Save the multiprocess arrays given the paths."""
   mpa_list, mpa_subpaths = zip(*mpa_targets)
@@ -211,6 +223,7 @@ def _save_mpas(gda_manager, mpa_targets: List[Tuple[MultiprocessArrayType, str]]
           keep,
           overwrite,
           keep_every_n_steps,
+          ckpt_start_time,
           has_mpa=True,
           write_commit_success=write_commit_success,
           async_manager=async_manager))
@@ -232,11 +245,11 @@ def _restore_mpas(state_dict,
 
   def _safe_deserialize(
       target_mpas: List[Tuple[Tuple[Any, ...], MultiprocessArrayType, str]],
-      gda_manager: GlobalAsyncCheckpointManager) -> List[MultiprocessArrayType]:
+      gda_manager: Any) -> List[MultiprocessArrayType]:
     gda_manager.wait_until_finished()
 
     # Check if reading from GCS and the array dir is potentially corrupted.
-    if ckpt_path.startswith('gs://') and not gfile.exists(
+    if ckpt_path.startswith('gs://') and not io.exists(
         os.path.join(ckpt_path + MP_ARRAY_POSTFIX, COMMIT_SUCCESS_FILE)):
       raise errors.MPARestoreDataCorruptedError(step, ckpt_path)
 
@@ -248,7 +261,7 @@ def _restore_mpas(state_dict,
       if isinstance(arr, GlobalDeviceArray) and jax.config.jax_array:
         raise errors.MPARestoreTypeNotMatchError(step, path)
       if isinstance(arr, GlobalDeviceArray):
-        shardings.append(sharding.MeshPspecSharding(arr.mesh, arr.mesh_axes))
+        shardings.append(sharding.NamedSharding(arr.mesh, arr.mesh_axes))
       elif jax.config.jax_array and isinstance(arr, jax.Array):
         shardings.append(arr.sharding)
 
@@ -324,17 +337,20 @@ def natural_sort(file_list: Iterable[str], signed: bool = True) -> List[str]:
 
 
 def safe_normpath(path: str) -> str:
-  """Normalizes path safely to get around `gfile.glob()` limitations."""
-  d = SCHEME_RE.match(path).groupdict()
+  """Normalizes path safely to get around `io.glob()` limitations."""
+  match = SCHEME_RE.match(path)
+  assert match is not None
+  d = match.groupdict()
   return (d['scheme'] or '') + os.path.normpath(d['path'])
 
 
 def _remove_invalid_ckpts(ckpt_path: str, base_path: str, keep: int,
                           overwrite: bool, keep_every_n_steps: Optional[int],
                           has_mpa: bool) -> None:
-  """Check the parameters and clean up the checkpoint space accordingly."""
+  """Clean up the checkpoint space according to `overwrite`, `keep`, and `keep_every_n_steps` parameters.
+  """
   dir_path, prefix = os.path.split(base_path)
-  checkpoint_files = [pathlib.PurePath(c) for c in gfile.listdir(dir_path)]
+  checkpoint_files: List[Any] = [pathlib.PurePath(c) for c in io.listdir(dir_path)]
   checkpoint_files = [
       os.path.join(dir_path, c)
       for c in checkpoint_files
@@ -343,7 +359,7 @@ def _remove_invalid_ckpts(ckpt_path: str, base_path: str, keep: int,
   checkpoint_files = natural_sort(checkpoint_files)
 
   # Remove newer checkpoints
-  if overwrite:
+  if overwrite and ckpt_path in checkpoint_files:
     ind = checkpoint_files.index(ckpt_path) + 1
     newer_ckpts = checkpoint_files[ind:]
     checkpoint_files = checkpoint_files[:ind]
@@ -353,9 +369,9 @@ def _remove_invalid_ckpts(ckpt_path: str, base_path: str, keep: int,
         # MPA might be removed already but the main ckpt is still there. This
         # can happen if the job is previously preempted after deleting the MPA
         # checkpoint folder and before deleting the main checkpoint.
-        if gfile.exists(path + MP_ARRAY_POSTFIX):
-          gfile.rmtree(path + MP_ARRAY_POSTFIX)
-      gfile.remove(path)
+        if io.exists(path + MP_ARRAY_POSTFIX):
+          io.rmtree(path + MP_ARRAY_POSTFIX)
+      _safe_remove(path)
 
   # Remove old checkpoint files.
   last_kept = -float('inf')
@@ -374,14 +390,15 @@ def _remove_invalid_ckpts(ckpt_path: str, base_path: str, keep: int,
       logging.info('Removing checkpoint at %s', path)
       if has_mpa:
         # MPA might be removed already but the main ckpt is still there.
-        if gfile.exists(path + MP_ARRAY_POSTFIX):
-          gfile.rmtree(path + MP_ARRAY_POSTFIX)
-      gfile.remove(path)
+        if io.exists(path + MP_ARRAY_POSTFIX):
+          io.rmtree(path + MP_ARRAY_POSTFIX)
+      _safe_remove(path)
 
 
 def _save_commit(ckpt_tmp_path: str, ckpt_path: str, base_path: str, keep: int,
                  overwrite: bool, keep_every_n_steps: Optional[int],
-                 has_mpa: bool, write_commit_success: bool,
+                 ckpt_start_time: float, has_mpa: bool,
+                 write_commit_success: bool,
                  async_manager: Optional[AsyncManager] = None) -> None:
   """Commit changes after saving checkpoints to disk.
 
@@ -391,41 +408,47 @@ def _save_commit(ckpt_tmp_path: str, ckpt_path: str, base_path: str, keep: int,
     2. Remove newer checkpoints (files that ordered larger than this save) if
     `overwrite=True`.
     3. Remove old checkpoint files based on `keep` and `keep_every_n_steps`.
+    4. Record program duration saved by this checkpoint.
   """
   mpa_ckpt_tmp_path, mpa_ckpt_path = ckpt_tmp_path + MP_ARRAY_POSTFIX, ckpt_path + MP_ARRAY_POSTFIX
   # Rename the multiprocess array path once serialization and writing finished.
   if has_mpa:
     if write_commit_success:
       commit_success_path = os.path.join(mpa_ckpt_path, COMMIT_SUCCESS_FILE)
-      with gfile.GFile(commit_success_path, 'w') as f:
+      with io.GFile(commit_success_path, 'w') as f:
         f.write(f'Checkpoint commit was successful to {mpa_ckpt_path}')
     else:
       # Commits are a two stage process (renaming the array folder and renaming
       # the main ckpt file in sequential order). We always try to overwrite
       # here because the array ckpt might be already renamed in a previously
-      # interrupted commit. NOTE: gfile.rename does not support overwriting
+      # interrupted commit. NOTE: io.rename does not support overwriting
       # directories via `rename` so we manually overwrite it.
-      if gfile.exists(mpa_ckpt_path):
+      if io.exists(mpa_ckpt_path):
         logging.info('Removing outdated checkpoint at %s', mpa_ckpt_path)
-        gfile.rmtree(mpa_ckpt_path)
-      gfile.rename(mpa_ckpt_tmp_path, mpa_ckpt_path)
+        io.rmtree(mpa_ckpt_path)
+      io.rename(mpa_ckpt_tmp_path, mpa_ckpt_path)
   # Commit the main checkpoint file after arrays (if any) are committed
   if async_manager:
     async_manager.wait_previous_save()
-  gfile.rename(ckpt_tmp_path, ckpt_path, overwrite=overwrite)
+  io.rename(ckpt_tmp_path, ckpt_path, overwrite=overwrite)
   logging.info('Saved checkpoint at %s', ckpt_path)
 
   # Remove newer and older invalid checkpoints.
   _remove_invalid_ckpts(ckpt_path, base_path, keep, overwrite,
                         keep_every_n_steps, has_mpa)
-
+  # Record checkpoint-related metrics.
+  orbax.utils.record_saved_duration(ckpt_start_time)
+  if async_manager:
+    jax.monitoring.record_event_duration_secs(
+        '/jax/checkpoint/write/async/total_duration_secs',
+        time.time() - ckpt_start_time)
 
 
 def _check_overwrite_error(ckpt_tmp_path: str, ckpt_path: str, base_path: str,
                            step: int):
   """Throw error if a ckpt file of this step or higher already exists."""
   dir_path, prefix = os.path.split(base_path)
-  checkpoint_files = [pathlib.PurePath(c) for c in _allowempty_listdir(dir_path)]
+  checkpoint_files: List[Any] = [pathlib.PurePath(c) for c in _allowempty_listdir(dir_path)]
   checkpoint_files = [
       os.path.join(dir_path, c)
       for c in checkpoint_files
@@ -447,12 +470,13 @@ def _check_overwrite_error(ckpt_tmp_path: str, ckpt_path: str, base_path: str,
 def _save_main_ckpt_file(target: bytes, has_mpa: bool, paths: Tuple[str, str],
                          base_path: str, step: int,
                          keep: int, overwrite: bool,
-                         keep_every_n_steps: Optional[int]):
+                         keep_every_n_steps: Optional[int],
+                         ckpt_start_time: float):
   """Save the main checkpoint file via file system."""
   ckpt_tmp_path, ckpt_path = paths
-  gfile.makedirs(os.path.dirname(ckpt_path))
+  io.makedirs(os.path.dirname(ckpt_path))
 
-  with gfile.GFile(ckpt_tmp_path, 'wb') as fp:
+  with io.GFile(ckpt_tmp_path, 'wb') as fp:
     fp.write(target)
 
   # Postpone the commitment of checkpoint to after MPA writes are done.
@@ -464,6 +488,7 @@ def _save_main_ckpt_file(target: bytes, has_mpa: bool, paths: Tuple[str, str],
         keep,
         overwrite,
         keep_every_n_steps,
+        ckpt_start_time,
         has_mpa=False,
         write_commit_success=False)
 
@@ -475,7 +500,7 @@ def _get_checkpoint_paths(
   """Generate the checkpoint paths used in this save operation."""
   ckpt_dir = os.fspath(ckpt_dir)  # Pathlib -> str
   logging.info('Saving checkpoint at step: %s', step)
-  # normalize path because gfile.glob() can modify path './', '//' ...
+  # normalize path because io.glob() can modify path './', '//' ...
   ckpt_dir = safe_normpath(ckpt_dir)
   ckpt_tmp_path = _checkpoint_path(ckpt_dir, 'tmp', prefix)
   ckpt_path = _checkpoint_path(ckpt_dir, step, prefix)
@@ -490,7 +515,8 @@ def save_checkpoint(ckpt_dir: Union[str, os.PathLike],
                     keep: int = 1,
                     overwrite: bool = False,
                     keep_every_n_steps: Optional[int] = None,
-                    async_manager: Optional[AsyncManager] = None) -> str:
+                    async_manager: Optional[AsyncManager] = None,
+                    orbax_checkpointer: Optional[orbax.Checkpointer] = None) -> str:
   """Save a checkpoint of the model. Suitable for single-host.
 
   In this method, every JAX process saves the checkpoint on its own. Do not
@@ -517,41 +543,76 @@ def save_checkpoint(ckpt_dir: Union[str, os.PathLike],
     async_manager: if defined, the save will run without blocking the main
       thread. Only works for single host. Note that an ongoing save will still
       block subsequent saves, to make sure overwrite/keep logic works correctly.
+    orbax_checkpointer: if defined, the save will be done by Orbax. In the
+      future, all Flax checkpointing features will be migrated to Orbax,
+      and starting to use an `orbax_checkpointer` is recommended. Please
+      check out the checkpointing guide (https://flax.readthedocs.io/en/latest/guides/use_checkpointing.html#save-checkpoints) for how to use Orbax checkpointers.
   Returns:
     Filename of saved checkpoint.
   """
+  start_time = time.time()
   # Make sure all saves are finished before the logic of checking and removing
   # outdated checkpoints happens.
   if async_manager:
     async_manager.wait_previous_save()
 
-  target = serialization.to_bytes(target)
-
   ckpt_path, ckpt_tmp_path, base_path = _get_checkpoint_paths(
       ckpt_dir, step, prefix)
+
+  if config.flax_use_orbax_checkpointing or orbax_checkpointer:
+    if jax.process_count() > 1:
+      logging.warning(
+          'Multiple JAX processes detected when saving checkpoint. Please note that only process 0 will execute the save, and you may lose data if the checkpoints saved by other processes contain unique data.'
+      )
+    # Make sure any previous work is done before making file changes.
+    if orbax_checkpointer and isinstance(orbax_checkpointer,
+                                         orbax.AsyncCheckpointer):
+      orbax_checkpointer.wait_until_finished()
+    if not orbax_checkpointer:
+      # If no checkpointer provided, save synchronously with default setting.
+      orbax_checkpointer = orbax.Checkpointer(orbax.PyTreeCheckpointHandler())
+    save_args = jax.tree_util.tree_map(lambda x: orbax.SaveArgs(aggregate=True),
+                                       target)
+    orbax_checkpointer.save(
+        ckpt_path, target, save_args=save_args, force=overwrite)
+    _remove_invalid_ckpts(ckpt_path, base_path, keep, overwrite,
+                          keep_every_n_steps, True)
+    end_time = time.time()
+    monitoring.record_event_duration_secs(_WRITE_CHECKPOINT_EVENT,
+                                          end_time - start_time)
+    return ckpt_path
+
   if not overwrite:
-    _check_overwrite_error(ckpt_tmp_path, ckpt_path, base_path, step)
+    _check_overwrite_error(ckpt_tmp_path, ckpt_path, base_path, step) # type: ignore
+
+  target = serialization.to_bytes(target)
   # Save the files via I/O sync or async.
   def save_main_ckpt_task():
+    jax.monitoring.record_event('/jax/flax/checkpoint/save_main_ckpt_task')
     return _save_main_ckpt_file(target, False, (ckpt_tmp_path, ckpt_path),
-                                base_path, step, keep,
-                                overwrite, keep_every_n_steps)
+                                base_path, step, keep, overwrite,
+                                keep_every_n_steps, start_time)
   if async_manager:
     async_manager.save_async(save_main_ckpt_task)
   else:
     save_main_ckpt_task()
+  end_time = time.time()
+  monitoring.record_event_duration_secs(_WRITE_CHECKPOINT_EVENT,
+                                        end_time - start_time)
   return ckpt_path
 
 
-def save_checkpoint_multiprocess(ckpt_dir: Union[str, os.PathLike],
-                                 target: PyTree,
-                                 step: Union[int, float],
-                                 prefix: str = 'checkpoint_',
-                                 keep: int = 1,
-                                 overwrite: bool = False,
-                                 keep_every_n_steps: Optional[int] = None,
-                                 async_manager: Optional[AsyncManager] = None,
-                                 gda_manager: Optional[Any] = None) -> str:
+def save_checkpoint_multiprocess(
+    ckpt_dir: Union[str, os.PathLike],
+    target: PyTree,
+    step: Union[int, float],
+    prefix: str = 'checkpoint_',
+    keep: int = 1,
+    overwrite: bool = False,
+    keep_every_n_steps: Optional[int] = None,
+    async_manager: Optional[AsyncManager] = None,
+    gda_manager: Optional[Any] = None,
+    orbax_checkpointer: Optional[orbax.Checkpointer] = None) -> str:
   """Save a checkpoint of the model in multi-process environment.
 
   Use this method to save `GlobalDeviceArray`s, or to save data to a
@@ -582,10 +643,15 @@ def save_checkpoint_multiprocess(ckpt_dir: Union[str, os.PathLike],
       correctly). Will save the GDAs to a separate subdirectory with postfix
       "_gda" asynchronously. Same as async_manager, this will block subsequent
       saves.
+    orbax_checkpointer: if defined, the save will be done by Orbax. In the
+      future, all Flax checkpointing features will be migrated to Orbax,
+      and starting to use an `orbax_checkpointer` is recommended. Please
+      check out the checkpointing guide (https://flax.readthedocs.io/en/latest/guides/use_checkpointing.html#save-checkpoints) for how to use Orbax checkpointers.
 
   Returns:
     Filename of saved checkpoint.
   """
+  start_time = time.time()
   # Make sure all saves are finished before the logic of checking and removing
   # outdated checkpoints happens.
   sync_global_devices('starting_save_checkpoint')
@@ -595,21 +661,44 @@ def save_checkpoint_multiprocess(ckpt_dir: Union[str, os.PathLike],
     gda_manager.wait_until_finished()
     sync_global_devices('before_save_checkpoint')
 
+  ckpt_path, ckpt_tmp_path, base_path = _get_checkpoint_paths(
+      ckpt_dir, step, prefix)
+
+  if config.flax_use_orbax_checkpointing or orbax_checkpointer:
+    # Make sure any previous work is done before making file changes.
+    if orbax_checkpointer and isinstance(orbax_checkpointer,
+                                         orbax.AsyncCheckpointer):
+      orbax_checkpointer.wait_until_finished()
+    # If no checkpointer provided, save synchronously with default setting.
+    if not orbax_checkpointer:
+      orbax_checkpointer = orbax.Checkpointer(orbax.PyTreeCheckpointHandler())
+    if process_index() == 0:
+      _remove_invalid_ckpts(ckpt_path, base_path, keep, overwrite,
+                            keep_every_n_steps, True)
+    save_args = jax.tree_util.tree_map(
+        lambda x: orbax.SaveArgs(
+            aggregate=not _use_multiprocess_serialization(x)), target)
+    orbax_checkpointer.save(
+        ckpt_path, target, save_args=save_args, force=overwrite)
+    end_time = time.time()
+    monitoring.record_event_duration_secs(_WRITE_CHECKPOINT_EVENT,
+                                          end_time - start_time)
+    return ckpt_path
+
   target = serialization.to_state_dict(target)
   target, mpa_targets = _split_mp_arrays(target)
   target = serialization.msgpack_serialize(target)
   has_mpa = mpa_targets and _IMPORT_GDAM_SUCCESSFUL
 
-  ckpt_path, ckpt_tmp_path, base_path = _get_checkpoint_paths(
-      ckpt_dir, step, prefix)
   if not overwrite:
-    _check_overwrite_error(ckpt_tmp_path, ckpt_path, base_path, step)
+    _check_overwrite_error(ckpt_tmp_path, ckpt_path, base_path, step) # type: ignore
     sync_global_devices('check_overwrite_strictly_before_save')
   # Save the files via I/O sync or async.
   def save_main_ckpt_task():
+    jax.monitoring.record_event('/jax/flax/checkpoint/save_main_ckpt_task')
     return _save_main_ckpt_file(target, has_mpa, (ckpt_tmp_path, ckpt_path),
-                                base_path, step, keep,
-                                overwrite, keep_every_n_steps)
+                                base_path, step, keep, overwrite,
+                                keep_every_n_steps, start_time)
   # Write the main checkpoint file only via process 0, to avoid race condition.
   if process_index() == 0:
     if async_manager:
@@ -626,8 +715,11 @@ def save_checkpoint_multiprocess(ckpt_dir: Union[str, os.PathLike],
       _make_mpa_dirs(mpa_targets, ckpt_tmp_path)
     sync_global_devices('Flax:Checkpointing:AfterCreateMPADir')
     _save_mpas(gda_manager, mpa_targets, ckpt_tmp_path, ckpt_path, base_path,
-               keep, overwrite, keep_every_n_steps, async_manager)
+               keep, overwrite, keep_every_n_steps, start_time, async_manager)
 
+  end_time = time.time()
+  monitoring.record_event_duration_secs(_WRITE_CHECKPOINT_EVENT,
+                                        end_time - start_time)
   return ckpt_path
 
 
@@ -643,7 +735,9 @@ def latest_checkpoint(ckpt_dir: Union[str, os.PathLike],
     The latest checkpoint path or None if no checkpoints were found.
   """
   ckpt_dir = os.fspath(ckpt_dir)  # Pathlib -> str
-  checkpoint_files = [pathlib.PurePath(c) for c in _allowempty_listdir(ckpt_dir)]
+  checkpoint_files: List[Any] = [
+      pathlib.PurePath(c) for c in _allowempty_listdir(ckpt_dir)
+  ]
   checkpoint_files = [
       os.path.join(ckpt_dir, c)
       for c in checkpoint_files
@@ -664,7 +758,8 @@ def restore_checkpoint(
     prefix: str = 'checkpoint_',
     parallel: bool = True,
     gda_manager: Optional[Any] = None,
-    allow_partial_mpa_restoration: bool = False) -> PyTree:
+    allow_partial_mpa_restoration: bool = False,
+    orbax_checkpointer: Optional[orbax.Checkpointer] = None) -> PyTree:
   """Restore last/best checkpoint from checkpoints in path.
 
   Sorts the checkpoint files naturally, returning the highest-valued
@@ -680,8 +775,8 @@ def restore_checkpoint(
     ckpt_dir: str: checkpoint file or directory of checkpoints to restore from.
     target: matching object to rebuild via deserialized state-dict. If None, the
       deserialized state-dict is returned as-is.
-    step: int or float: step number to load or None to load latest. If specified,
-      ckpt_dir must be a directory.
+    step: int or float: step number to load or None to load latest. If
+      specified, ckpt_dir must be a directory.
     prefix: str: name prefix of checkpoint files.
     parallel: bool: whether to load seekable checkpoints in parallel, for speed.
     gda_manager: required if checkpoint contains a multiprocess array
@@ -694,6 +789,8 @@ def restore_checkpoint(
       may have some MPAs not restored correctly. Use this if you cannot provide
       a fully valid ``target`` and don't need all the MPAs in the checkpoint
       to be restored.
+    orbax_checkpointer: the `Orbax.Checkpointer` that handles the underlying
+      restore, if the given checkpoint is saved with Orbax.
 
   Returns:
     Restored `target` updated from checkpoint file, or if no step specified and
@@ -702,27 +799,59 @@ def restore_checkpoint(
     returned. This is to match the behavior of the case where a directory path
     is specified but the directory has not yet been created.
   """
+  start_time = time.time()
+  # Make sure any previous work is done before checking files.
+  if orbax_checkpointer and isinstance(orbax_checkpointer,
+                                       orbax.AsyncCheckpointer):
+    orbax_checkpointer.wait_until_finished()
+
   ckpt_dir = os.fspath(ckpt_dir)  # Pathlib -> str
   ckpt_dir = safe_normpath(ckpt_dir)
-  if step is not None:
+  if step:
     ckpt_path = _checkpoint_path(ckpt_dir, step, prefix)
-    if not gfile.exists(ckpt_path):
+    if not io.exists(ckpt_path):
       raise ValueError(f'Matching checkpoint not found: {ckpt_path}')
   else:
-    if not gfile.exists(ckpt_dir):
+    if not io.exists(ckpt_dir):
       logging.info('Found no checkpoint directory at %s', ckpt_dir)
       return target
-    if not gfile.isdir(ckpt_dir):
-      ckpt_path = ckpt_dir
+    if io.isdir(ckpt_dir):
+      # This means the given dir is an orbax checkpoint.
+      if io.exists(os.path.join(ckpt_dir, ORBAX_CKPT_FILENAME)):
+        ckpt_path = ckpt_dir
+      else:
+        ckpt_path = latest_checkpoint(ckpt_dir, prefix) # type: ignore
+        if not ckpt_path:
+          logging.info('Found no checkpoint files in %s with prefix %s',
+                       ckpt_dir, prefix)
+          return target
     else:
-      ckpt_path = latest_checkpoint(ckpt_dir, prefix)
-      if not ckpt_path:
-        logging.info('Found no checkpoint files in %s with prefix %s',
-                     ckpt_dir, prefix)
-        return target
+      ckpt_path = ckpt_dir
 
   logging.info('Restoring checkpoint from %s', ckpt_path)
-  with gfile.GFile(ckpt_path, 'rb') as fp:
+
+  # Restore the checkpoint with Orbax if needed.
+  is_orbax = io.exists(os.path.join(ckpt_path, ORBAX_CKPT_FILENAME))
+  if is_orbax:
+    if not orbax_checkpointer:
+      orbax_checkpointer = orbax.Checkpointer(orbax.PyTreeCheckpointHandler())
+
+    def make_restore_args(x):
+      if isinstance(x, GlobalDeviceArray):
+        return orbax.ArrayRestoreArgs(mesh=x.mesh, mesh_axes=x.mesh_axes)
+      return orbax.ArrayRestoreArgs()
+
+    restore_args = None
+    if target:
+      restore_args = jax.tree_util.tree_map(make_restore_args, target)
+    restored = orbax_checkpointer.restore(
+        ckpt_path, item=target, restore_args=restore_args)
+    end_time = time.time()
+    monitoring.record_event_duration_secs(_READ_CHECKPOINT_EVENT,
+                                          end_time - start_time)
+    return restored
+
+  with io.GFile(ckpt_path, 'rb') as fp:
     if parallel and fp.seekable():
       buf_size = 128 << 20  # 128M buffer.
       num_bufs = fp.size() / buf_size
@@ -733,7 +862,7 @@ def restore_checkpoint(
         # NOTE: We have to re-open the file to read each chunk, otherwise the
         # parallelism has no effect. But we could reuse the file pointers
         # within each thread.
-        with gfile.GFile(ckpt_path, 'rb') as f:
+        with io.GFile(ckpt_path, 'rb') as f:
           f.seek(i * buf_size)
           buf = f.read(buf_size)
           if buf:
@@ -754,8 +883,15 @@ def restore_checkpoint(
                                allow_partial_mpa_restoration)
 
   if target is None:
-    return state_dict
-  return serialization.from_state_dict(target, state_dict)
+    restored_checkpoint = state_dict
+  else:
+    restored_checkpoint = serialization.from_state_dict(target, state_dict)
+
+  end_time = time.time()
+  monitoring.record_event_duration_secs(_READ_CHECKPOINT_EVENT,
+                                        end_time - start_time)
+
+  return restored_checkpoint
 
 
 def convert_pre_linen(params: PyTree) -> PyTree:
@@ -811,7 +947,7 @@ def convert_pre_linen(params: PyTree) -> PyTree:
   if not isinstance(params, (dict, core.FrozenDict)):
     return params
   params_renamed = {}
-  counts = {}
+  counts: Dict[Any, Any] = {}
   names = natural_sort(params.keys())
   for name in names:
     value = params[name]
@@ -824,5 +960,5 @@ def convert_pre_linen(params: PyTree) -> PyTree:
     params_renamed[name] = convert_pre_linen(value)
 
   if isinstance(params, core.FrozenDict):
-    params_renamed = core.freeze(params_renamed)
+    params_renamed = core.freeze(params_renamed) # type: ignore
   return params_renamed
